// WaveFile.cpp : Cve[V t@C
//

#include "stdafx.h"
#include "SpoolWave.h"
#include "WaveFile.h"
#include "WaitStatus.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWaveFile

CWaveFile::CWaveFile(CWnd *Wnd)
{
	int n;

	pWnd = Wnd;
	hmmio = NULL;
	hWaveOut = NULL;
	fileName = "";

	memset(&waveFormat, 0, sizeof(WAVEFORMATEX));
	waveSize = 0L;
	maxLevel = (-1);
	volLevel = MAXVOLUME;
	FadeIn = FadeOut = 0;
	recLevel = 0;

	for ( n = 0 ; n < MAXWAVEHDR ; n++ )
		waveHdrTab[n].lpData = NULL;

	waveFormat.wFormatTag      = WAVE_FORMAT_PCM;
    waveFormat.nChannels       = 2;
    waveFormat.nSamplesPerSec  = 44100;
    waveFormat.nBlockAlign     = 2 * (16 / 8);
    waveFormat.nAvgBytesPerSec = 44100 * waveFormat.nBlockAlign;
    waveFormat.wBitsPerSample  = 16;
    waveFormat.cbSize = 0;
}

CWaveFile::~CWaveFile()
{
	int n;

	if ( hWaveOut != NULL ) {
		waveOutReset(hWaveOut);
		for ( n = 0 ; n < MAXWAVEHDR ; n++ ) {
			if ( waveHdrTab[n].lpData != NULL )
				waveOutUnprepareHeader(hWaveOut, &(waveHdrTab[n]), sizeof(WAVEHDR));
		}
		waveOutClose(hWaveOut);
	}

	if ( hWaveIn != NULL ) {
	    waveInStop(hWaveIn);
		waveInReset(hWaveIn);
		for ( n = 0 ; n < MAXWAVEHDR ; n++ ) {
			if ( waveHdrTab[n].lpData != NULL )
			waveInUnprepareHeader(hWaveIn, &(waveHdrTab[n]), sizeof(WAVEHDR));
		}
		waveInClose(hWaveIn);
	}

	if ( hmmio != NULL )
		mmioClose(hmmio, 0);
}

BEGIN_MESSAGE_MAP(CWaveFile, CWnd)
	//{{AFX_MSG_MAP(CWaveFile)
		//  - ClassWizard ͂̈ʒuɃ}bsOp̃}Nǉ܂͍폜܂B
        ON_MESSAGE(MM_WOM_DONE, OnWaveOutData)
        ON_MESSAGE(MM_WIM_DATA, OnWaveInData)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWaveFile bZ[W nh

int CWaveFile::OpenFile(LPCSTR file)
{
    PCMWAVEFORMAT pcmWave;

	if ( (hmmio = mmioOpen((LPSTR)file, NULL, MMIO_READ | MMIO_ALLOCBUF)) == NULL )
		return FALSE;

	mmckinfoParent.fccType = mmioFOURCC('W', 'A', 'V', 'E');
	if ( mmioDescend(hmmio, (LPMMCKINFO)(&mmckinfoParent), NULL, MMIO_FINDRIFF) )
		goto ERRORRET;

	mmckinfoSubchunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
	if ( mmioDescend(hmmio, &mmckinfoSubchunk, &mmckinfoParent, MMIO_FINDCHUNK) )
		goto ERRORRET;

	if ( mmioRead(hmmio, (LPSTR)&pcmWave, mmckinfoSubchunk.cksize) != (LRESULT)mmckinfoSubchunk.cksize )
		goto ERRORRET;

	if ( pcmWave.wf.wFormatTag != WAVE_FORMAT_PCM )
		goto ERRORRET;

	mmioAscend(hmmio, &mmckinfoSubchunk, 0);
	mmckinfoSubchunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
	if ( mmioDescend(hmmio, &mmckinfoSubchunk, &mmckinfoParent, MMIO_FINDCHUNK) )
		goto ERRORRET;

	if ( mmckinfoSubchunk.cksize == 0 )
		goto ERRORRET;

    waveFormat.wFormatTag      = pcmWave.wf.wFormatTag;
    waveFormat.nChannels       = pcmWave.wf.nChannels;
    waveFormat.nSamplesPerSec  = pcmWave.wf.nSamplesPerSec;
	waveFormat.nBlockAlign     = pcmWave.wf.nBlockAlign;
    waveFormat.nAvgBytesPerSec = pcmWave.wf.nAvgBytesPerSec;
	waveFormat.wBitsPerSample  = pcmWave.wBitsPerSample;
    waveFormat.cbSize          = 0;
	waveSize = mmckinfoSubchunk.cksize;
	playSize = 0;

	return TRUE;

ERRORRET:
    mmioClose(hmmio, 0);
	hmmio = NULL;
    return FALSE;
}

void CWaveFile::CloseFile()
{
	if ( hmmio != NULL )
	    mmioClose(hmmio, 0);
	hmmio = NULL;
}

int CWaveFile::InitFile(LPCSTR file)
{
	fileName = file;

	if ( !OpenFile(fileName) )
		return FALSE;

	CloseFile();

	return TRUE;
}

int CWaveFile::PlayStart()
{
    int n, i;
    WAVEHDR *pwh;
    RECT rect;

	if ( !OpenFile(fileName) )
		return FALSE;

    memset(&rect, 0, sizeof(RECT));
    if ( !Create(NULL, "WaveFile", WS_CHILD, rect, pWnd, 0, NULL) ) {
		CloseFile();
		return FALSE;
	}
    
	if ( waveOutOpen(&hWaveOut, WAVE_MAPPER, &waveFormat, (DWORD)(m_hWnd), 0L, CALLBACK_WINDOW)) {
		CloseFile();
		DestroyWindow();
		return FALSE;
	}

	playSize = 0;
	stopFlag = FALSE;

    for ( n = 0 ; n < MAXWAVEHDR ; n++ ) {
		pwh = &(waveHdrTab[n]);
        memset(pwh, 0, sizeof(WAVEHDR));
		pwh->lpData          = waveBuffer[n];
		pwh->dwBufferLength  = MAXWAVEBUF;
		pwh->dwBytesRecorded = 0;
		pwh->dwFlags         = 0;
		pwh->dwLoops         = 0;
		pwh->dwUser          = n;
		if( waveOutPrepareHeader(hWaveOut, pwh, sizeof(WAVEHDR))) {
			pwh->lpData = NULL;
		    goto ERRORRET;
		}
		if ( playSize < waveSize ) {
			if ( (i = (waveSize - playSize)) > (int)(pwh->dwBufferLength) )
				i = pwh->dwBufferLength;
			if( (i = mmioRead(hmmio, pwh->lpData, i)) > 0 ) {
				pwh->dwBufferLength  = i;
				VolumeCalc(playSize, i, pwh->lpData);
				if ( waveOutWrite(hWaveOut, pwh, sizeof(WAVEHDR)) )
					goto ERRORRET;
				playSize += i;
			}
		}
	}

	return TRUE;

ERRORRET:
	CloseFile();
	waveOutReset(hWaveOut);
    for ( n = 0 ; n < MAXWAVEHDR ; n++ ) {
		if ( waveHdrTab[n].lpData != NULL )
			waveOutUnprepareHeader(hWaveOut, &(waveHdrTab[n]), sizeof(WAVEHDR));
		waveHdrTab[n].lpData = NULL;
	}
    waveOutClose(hWaveOut);
	hWaveOut = NULL;
	DestroyWindow();
    return FALSE;
}

int CWaveFile::PlayStatus()
{
	int n;

	if ( hWaveOut == NULL )
		return FALSE;
    for ( n = 0 ; n < MAXWAVEHDR ; n++ ) {
		if ( waveHdrTab[n].lpData != NULL )
			return TRUE;
	}
	return FALSE;
}

void CWaveFile::PlayEnd()
{
	int n;

	stopFlag = TRUE;
	if ( hWaveOut != NULL ) {
		waveOutReset(hWaveOut);
		for ( n = 0 ; n < MAXWAVEHDR ; n++ ) {
			if ( waveHdrTab[n].lpData != NULL )
				waveOutUnprepareHeader(hWaveOut, &(waveHdrTab[n]), sizeof(WAVEHDR));
			waveHdrTab[n].lpData = NULL;
		}
		waveOutClose(hWaveOut);
		hWaveOut = NULL;
	}
	CloseFile();
	DestroyWindow();
}

void CWaveFile::OnWaveOutData(WPARAM wParam, LPARAM lParam)
{
	int i;
   WAVEHDR *pwh = (WAVEHDR *)lParam;

   if ( stopFlag ) {
		if ( pwh->lpData != NULL )
			waveOutUnprepareHeader(hWaveOut, pwh, sizeof(WAVEHDR));
		pwh->lpData = NULL;
		return;
   }

	pwh->dwFlags = 0;
	pwh->dwLoops = 0;

	if ( (waveSize - playSize) > MAXWAVEBUF )
		i = MAXWAVEBUF;
	else {
		i = (int)(waveSize - playSize);
		stopFlag = TRUE;
	}

	if( (i = mmioRead(hmmio, pwh->lpData, i)) > 0 ) {
		pwh->dwBufferLength  = i;
		VolumeCalc(playSize, i, pwh->lpData);
		if ( waveOutWrite(hWaveOut, pwh, sizeof(WAVEHDR)) )
			return;
		playSize += i;
	}
}

void CWaveFile::VolumeCalc(DWORD pos, int len, char *buff)
{
	int n, vol, ext;
	int mx = (len / waveFormat.nBlockAlign) * waveFormat.nChannels;
	short *sp = (short *)buff;
	unsigned char *cp = (unsigned char *)buff;
	DWORD fmx  = waveSize / waveFormat.nBlockAlign;
	DWORD fin  = waveFormat.nSamplesPerSec * FadeIn;
	DWORD fout = waveFormat.nSamplesPerSec * FadeOut;

	pos /= waveFormat.nBlockAlign;
	if ( pos < fin )
		ext = volLevel * pos / fin;
	else if ( pos > (fmx - fout) )
		ext = volLevel * (fmx - pos) / fout;
	else
		ext = volLevel;

	if ( ext == MAXVOLUME )
		return;

	switch(waveFormat.wBitsPerSample) {
	case 8:
		for ( n = 0 ; n < mx ; n++ ) {
			vol = *cp - 128;
			vol = vol * ext / MAXVOLUME;

			if ( vol > 127 )
				vol = 127;
			else if ( vol < -128 )
				vol = -128;

			*(cp++) = (unsigned char)(vol + 128);
		}
		break;
	case 16:
		for ( n = 0 ; n < mx ; n++ ) {
			vol = *sp;
			vol = vol * ext / MAXVOLUME;

			if ( vol > 32767 )
				vol = 32767;
			else if ( vol < -32768 )
				vol = -32768;
			
			*(sp++) = (short)vol;
		}
		break;
	}
}

int CWaveFile::MaxLevel()
{
	int n, mx, vol;
	DWORD size;
	char tmp[4096];
	short *sp;
	unsigned char *cp;
	CWaitStatus status(this);

	if ( maxLevel >= 0 )
		return maxLevel;

	if ( !OpenFile(fileName) )
		return (-1);

	status.m_Title = fileName;

	if ( !status.Create(IDD_WAIT_STATUS, this) )
		return (-1);

	status.m_Progress.SetRange32(0, waveSize);
	status.ShowWindow(SW_SHOW);
	status.UpdateWindow();

	maxLevel = 0;

	for ( size = 0 ; size < waveSize ; size += 4096 ) {
		if ( (waveSize - size) > 4096 )
			n = 4096;
		else
			n = (int)(waveSize - size);

		if( (n = mmioRead(hmmio, tmp, n)) <= 0 )
			break;

		mx = (n / waveFormat.nBlockAlign) * waveFormat.nChannels;

		switch(waveFormat.wBitsPerSample) {
		case 8:
			cp = (unsigned char *)tmp;
			for ( n = 0 ; n < mx ; n++ ) {
				if ( (vol = GetLevel(*(cp++) - 128)) > maxLevel )
					maxLevel = vol;
			}
			break;
		case 16:
			sp = (short *)tmp;
			for ( n = 0 ; n < mx ; n++ ) {
				if ( (vol = GetLevel(*(sp++))) > maxLevel )
					maxLevel = vol;
			}
			break;
		}

		status.m_Progress.SetPos(size);

		MSG msg;
	    while ( PeekMessage(&msg, 0, 0, 0, PM_REMOVE) ) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	CloseFile();
	return maxLevel;
}

void CWaveFile::RegLevel()
{
	int max = 1 << (waveFormat.wBitsPerSample - 1);

	if ( MaxLevel() <= 0 )
		return;

	volLevel = MAXVOLUME * max / maxLevel;
}

int CWaveFile::SaveFile()
{
	int n;
	HMMIO hnewio;
	CString file;
	MMCKINFO newckinfoParent;
	MMCKINFO newckinfoSubchunk;
    PCMWAVEFORMAT pcmWave;
	DWORD sz;
	char tmp[MAXWAVEBUF];
	CWaitStatus status(this);

	if ( volLevel == MAXVOLUME && FadeIn == 0 && FadeOut == 0 )
		return TRUE;

	file = fileName;
	file += ".$$$";

	if ( PlayStatus() )
		PlayEnd();

	if ( !OpenFile(fileName) )
		return FALSE;

    if( (hnewio = mmioOpen((LPSTR)((LPCSTR)file), NULL, MMIO_WRITE | MMIO_CREATE)) == NULL)
		return FALSE;

    memset(&newckinfoParent, 0, sizeof(MMCKINFO));
    memset(&newckinfoSubchunk, 0, sizeof(MMCKINFO));

    newckinfoParent.fccType = mmioFOURCC('W', 'A', 'V', 'E');
    if ( mmioCreateChunk(hnewio, &newckinfoParent, MMIO_CREATERIFF) )
		goto ERRORRET;

    newckinfoSubchunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
    newckinfoSubchunk.cksize = sizeof(PCMWAVEFORMAT);
    if ( mmioCreateChunk(hnewio, &newckinfoSubchunk, 0) )
		goto ERRORRET;

    pcmWave.wf.wFormatTag      = waveFormat.wFormatTag;
    pcmWave.wf.nChannels       = waveFormat.nChannels;
    pcmWave.wf.nSamplesPerSec  = waveFormat.nSamplesPerSec;
    pcmWave.wf.nBlockAlign     = waveFormat.nBlockAlign;
    pcmWave.wf.nAvgBytesPerSec = waveFormat.nAvgBytesPerSec;
    pcmWave.wBitsPerSample     = waveFormat.wBitsPerSample;

    if ( mmioWrite(hnewio, (LPCSTR)&pcmWave, sizeof(PCMWAVEFORMAT)) != sizeof(PCMWAVEFORMAT) )
		goto ERRORRET;

    mmioAscend(hnewio, &newckinfoSubchunk, 0);
    newckinfoSubchunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
    if ( mmioCreateChunk(hnewio, &newckinfoSubchunk, 0) )
		goto ERRORRET;

	status.m_Title = fileName;
	status.Create(IDD_WAIT_STATUS, this);
	status.m_Progress.SetRange32(0, waveSize);
	status.ShowWindow(SW_SHOW);
	status.UpdateWindow();

	for ( sz = 0 ; sz < waveSize ; sz += MAXWAVEBUF ) {
		if ( (waveSize - sz) > MAXWAVEBUF )
			n = MAXWAVEBUF;
		else
			n = (int)(waveSize - sz);

		if( (n = mmioRead(hmmio, tmp, n)) <= 0 )
			break;

		VolumeCalc(sz, n, tmp);

		if ( mmioWrite(hnewio, tmp, n) != n )
			break;

		status.m_Progress.SetPos(sz);

		MSG msg;
	    while ( PeekMessage(&msg, 0, 0, 0, PM_REMOVE) ) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	if ( sz < waveSize )
		goto ERRORRET;

	mmioAscend(hnewio, &newckinfoSubchunk, 0);
	mmioAscend(hnewio, &newckinfoParent, 0);
    mmioClose(hnewio, 0);
	CloseFile();
/*
	volLevel = MAXVOLUME;
	FadeIn = FadeOut = 0;
*/
	if ( unlink(fileName) || rename(file, fileName) )
		return FALSE;

	return TRUE;

ERRORRET:
	CloseFile();
    mmioClose(hnewio, 0);
	unlink(file);
    return FALSE;
}

int CWaveFile::CreateFile(LPCSTR file)
{
    PCMWAVEFORMAT pcmWave;

    if( (hmmio = mmioOpen((LPSTR)file, NULL, MMIO_WRITE | MMIO_CREATE)) == NULL)
		return FALSE;

    memset(&mmckinfoParent, 0, sizeof(MMCKINFO));
    memset(&mmckinfoSubchunk, 0, sizeof(MMCKINFO));

    mmckinfoParent.fccType = mmioFOURCC('W', 'A', 'V', 'E');
    if ( mmioCreateChunk(hmmio, &mmckinfoParent, MMIO_CREATERIFF) )
		goto ERRORRET;

    mmckinfoSubchunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
    mmckinfoSubchunk.cksize = sizeof(PCMWAVEFORMAT);
    if ( mmioCreateChunk(hmmio, &mmckinfoSubchunk, 0) )
		goto ERRORRET;

    pcmWave.wf.wFormatTag      = waveFormat.wFormatTag;
    pcmWave.wf.nChannels       = waveFormat.nChannels;
    pcmWave.wf.nSamplesPerSec  = waveFormat.nSamplesPerSec;
    pcmWave.wf.nBlockAlign     = waveFormat.nBlockAlign;
    pcmWave.wf.nAvgBytesPerSec = waveFormat.nAvgBytesPerSec;
    pcmWave.wBitsPerSample     = waveFormat.wBitsPerSample;

    if ( mmioWrite(hmmio, (LPCSTR)&pcmWave, sizeof(PCMWAVEFORMAT)) != sizeof(PCMWAVEFORMAT) )
		goto ERRORRET;

    mmioAscend(hmmio, &mmckinfoSubchunk, 0);
    mmckinfoSubchunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
    if ( mmioCreateChunk(hmmio, &mmckinfoSubchunk, 0) )
		goto ERRORRET;

	return TRUE;

ERRORRET:
    mmioClose(hmmio, 0);
	hmmio = NULL;
    return FALSE;
}

void CWaveFile::AscendFile()
{
	mmioAscend(hmmio, &mmckinfoSubchunk, 0);
	mmioAscend(hmmio, &mmckinfoParent, 0);
    mmioClose(hmmio, 0);
	hmmio = NULL;
}

int CWaveFile::ReadData(void *buff, int len)
{
	int n;

	if ( (waveSize - playSize) < (DWORD)len )
		len = (int)(waveSize - playSize);

	if ( (n = mmioRead(hmmio, (char *)buff, len)) > 0 )
		VolumeCalc(playSize, n, (char *)buff);

	playSize += len;
	return n;
}

int CWaveFile::WriteData(void *buff, int len)
{
	waveSize += len;
	return mmioWrite(hmmio, (char *)buff, len);
}

int CWaveFile::RecodeStart(LPCSTR file, int max)
{
    int n;
    WAVEHDR *pwh;
    RECT rect;

	if ( !tmpFile.Open(file, CFile::modeCreate | CFile::modeReadWrite | CFile::typeBinary) )
		return FALSE;

    memset(&rect, 0, sizeof(RECT));

    if ( !Create(NULL, "WaveRecord", WS_CHILD, rect, pWnd, 0, NULL) ) {
		tmpFile.Close();
		return FALSE;
	}
    
    if ( waveInOpen(&hWaveIn, WAVE_MAPPER, &waveFormat, (DWORD)(m_hWnd), 0L, CALLBACK_WINDOW) ) {
		DestroyWindow();
		tmpFile.Close();
		return FALSE;
	}

	waveSize = playSize = 0;
	recdSize = (DWORD)max * (DWORD)(waveFormat.nAvgBytesPerSec);
	stopFlag = FALSE;
	recLevel = 0;

    for ( n = 0 ; n < MAXWAVEHDR ; n++ ) {
		pwh = &(waveHdrTab[n]);
        memset(pwh, 0, sizeof(WAVEHDR));
		pwh->lpData          = waveBuffer[n];
		pwh->dwBufferLength  = MAXWAVEBUF;
		pwh->dwBytesRecorded = 0;
		pwh->dwFlags         = 0;
		pwh->dwLoops         = 0;
		pwh->dwUser          = n;
		if( waveInPrepareHeader(hWaveIn, pwh, sizeof(WAVEHDR)))
		    goto ERRORRET;
		if ( waveInAddBuffer(hWaveIn, pwh, sizeof(WAVEHDR)) )
			goto ERRORRET;
    }

    if( waveInStart(hWaveIn) )
		goto ERRORRET;

    return TRUE;

ERRORRET:
    waveInClose(hWaveIn);
	hWaveIn = NULL;
	DestroyWindow();
	tmpFile.Close();
    return FALSE;
}

int CWaveFile::RecodeEnd(LPCSTR file)
{
	int n;
	DWORD endSize = playSize;
	DWORD saveSize = 0;
	char tmp[MAXWAVEBUF];
	CString name;
	CWaitStatus status;

    stopFlag = TRUE;
	if ( hWaveIn != NULL ) {
	    waveInStop(hWaveIn);
		waveInReset(hWaveIn);
		for ( n = 0 ; n < MAXWAVEHDR ; n++ ) {
			if ( waveHdrTab[n].lpData != NULL )
			waveInUnprepareHeader(hWaveIn, &(waveHdrTab[n]), sizeof(WAVEHDR));
		}
		waveInClose(hWaveIn);
		hWaveIn = NULL;
	}
	tmpFile.Flush();
	DestroyWindow();

	if ( !CreateFile(file) )
		return FALSE;

	status.m_Title = file;
	status.Create(IDD_WAIT_STATUS, this);
	status.m_Progress.SetRange32(0, recdSize);
	status.ShowWindow(SW_SHOW);
	status.UpdateWindow();

	tmpFile.Seek(playSize, CFile::begin);
	do {
		if ( playSize >= recdSize ) {
			tmpFile.SeekToBegin();
			playSize = 0;
		}
		n = tmpFile.Read(tmp, MAXWAVEBUF);
		WriteData(tmp, n);
		playSize += MAXWAVEBUF;

		saveSize += MAXWAVEBUF;
		status.m_Progress.SetPos(saveSize);
		MSG msg;
	    while ( PeekMessage(&msg, 0, 0, 0, PM_REMOVE) ) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

	} while ( playSize != endSize && saveSize < recdSize );

	AscendFile();
	name = tmpFile.GetFilePath();
	tmpFile.Close();
	tmpFile.Remove(name);
	recLevel = 0;

	return TRUE;
}

void CWaveFile::OnWaveInData(WPARAM wParam, LPARAM lParam)
{
    int n, val, max;
	unsigned char *cp;
	short *sp;
	WAVEHDR *pwh = (WAVEHDR *)lParam;

    if ( waveInUnprepareHeader(hWaveIn, pwh, sizeof(WAVEHDR)) )
		goto ERRORRET;

	if ( stopFlag )
		goto ERRORRET;

	max = pwh->dwBytesRecorded / waveFormat.nBlockAlign * waveFormat.nChannels;

	switch(waveFormat.wBitsPerSample) {
	case 8:
		cp = (unsigned char *)(pwh->lpData);
		for ( n =  0 ; n < max ; n++ ) {
			if ( (val = GetLevel(*(cp++) - 128)) > recLevel )
				recLevel = val;
		}
		break;
	case 16:
		sp = (short *)(pwh->lpData);
		for ( n =  0 ; n < max ; n++ ) {
			if ( (val = GetLevel(*(sp++))) > recLevel )
				recLevel = val;
		}
		recLevel = recLevel / 256;
		break;
	}

	if ( playSize >= recdSize ) {
		tmpFile.Flush();
		tmpFile.SeekToBegin();
		playSize = 0;
	}

    if ( pwh->dwBytesRecorded > 0 ) {
		tmpFile.Write(pwh->lpData, pwh->dwBytesRecorded);
		playSize += pwh->dwBytesRecorded;
	}

	pwh->dwBytesRecorded = 0;
	pwh->dwFlags         = 0;
	pwh->dwLoops         = 0;

    if ( waveInPrepareHeader(hWaveIn, pwh, sizeof(WAVEHDR)) )
		goto ERRORRET;

    if ( waveInAddBuffer(hWaveIn, pwh, sizeof(WAVEHDR)) )
		goto ERRORRET;

	return;

ERRORRET:
	pwh->lpData = NULL;
}
